package org.dvb.application.privileged;

import org.dvb.application.AppAttributes;
import org.dvb.application.storage.ApplicationStorageController;
import org.davic.net.Locator;

/**
 * Enables privileged applications to make decisions for some methods in 
 * the org.dvb.application.storage package.
 * The methods in this interface correspond to the methods in ApplicationStorageController
 * that throw a UserRejectedInstallException. If an application storage handler 
 * does not permit a request then it should return false and the original method
 * shall fail with a UserRejectedInstallException.<p>
 * Implementations of the methods in this interface should complete quickly
 * and not perform time-consuming operations such as blocking I/O calls.
 */

public interface ApplicationStorageHandler {
	/**
	 * Called as part of the execution of ApplicationStorageController.store(AppProxy,...).
	 * @param service the service for which store was called
	 * @param app the application to be installed
	 * @param autoStart the parameter as passed in the call to the store method
	 * @param args an array of strings where each entry is .equals to
	 * the corresponding entry passed in the call to the store method
 	 * @return true if the storage request is permitted, otherwise false
	 */
	public boolean storeRequested(ApplicationStorageController service,
		AppAttributes app, boolean autoStart, java.lang.String[] args);

	/**
	 * Called as part of the execution of ApplicationStorageController.store(AppProxy[],...).
	 * @param service the service for which store was called
	 * @param app the applications to be installed
	 * @param autoStart the array of booleans as passed in the call to the store method
	 * @param args an copy of the corresponding parameter passed to the 
	 * store method where each entry is .equals to
	 * the corresponding entry passed in the call to the store method
 	 * @return true if the storage request is permitted, otherwise false
	 */
	public boolean storeRequested(ApplicationStorageController service,
		AppAttributes[] app, boolean[] autoStart, java.lang.String[][] args);

	/**
	 * Called as part of the execution of ApplicationStorageController.store(Locator,AppID[],...).
         * @param fromService the service containing the applications to be stored
	 * @param toService the service into which the applications are to be stored
	 * @param app the applications to be installed
	 * @param autoStart the array of booleans as passed in the call to the store method
	 * @param args an copy of the corresponding parameter passed to the 
	 * store method where each entry is .equals to
	 * the corresponding entry passed in the call to the store method
 	 * @return true if the storage request is permitted, otherwise false
	 */
	public boolean storeRequested(Locator fromService, ApplicationStorageController toService,
		AppAttributes[] app, boolean[] autoStart, java.lang.String[][] args);

	/**
	 * Called as part of the execution of ApplicationStorageController.removeService.
	 * @param service the service for which removeService was called
	 * @return true if the remove request is permitted, otherwise false
	 */
	public boolean removeServiceRequested(ApplicationStorageController service);

	/**
	 * Called as part of the execution of ApplicationStorageController.remove(AppID).
	 * @param app the application to be removed
	 * @return true if the remove request is permitted, otherwise false
	 */
	public boolean removeAppRequested(AppAttributes app);

	/**
	 * Called as part of the execution of ApplicationStorageController.remove(AppID[]).
	 * @param app the applications to be removed
	 * @return true if the remove request is permitted, otherwise false
	 */
	public boolean removeAppRequested(AppAttributes[] app);
}

