/**
 *
 */
package org.dvb.media;

import javax.media.Control;
import javax.media.Controller;
import javax.media.Player;
import javax.media.Time;
import javax.tv.service.selection.InsufficientResourcesException;

/**
 * MasterSlaveSyncLinkageControl handles Player-based Master/Slave relationship for synchronization
 * of JMF Players. A JMF Player that provides master clock for synchronization is a <B><I>master Player</I></B>.
 * MasterSlaveSyncLinkageControl allows a JMF Player from which this Control is obtained to be a master Player.
 * At the time MasterSlaveSyncLinkageControl object is obtained from a Controller object of a JMF Player,
 * the Player is a potential master Player. When a Controller object of a <B><I>slave Player</I></B> is added
 * this control by {@link #addSlave(Controller) addSlave()} method, actual synchronization relationship is
 * established. If synchronization between the Players is successfully established, a JMF Player to
 * which MasterSlaveSyncLinkageControl object belongs becomes a master Player. When all the slave Players
 * are removed from the MasterSlaveSyncLinkageControl object by {@link #removeSlave(Controller) removeSlave()}
 * method, a master Player is back to a potential master Player.
 * <BR><BR>
 *
 * MasterSlaveSyncLinkageControl object monitors synchronization status between the master
 * and slave Players. When synchronization status is changed, MasterSlaveSyncLinkageControl object
 * notifies the situation by throwing SyncStateChangedEvent to the application. Before adding a slave Player,
 * synchronization status is 'Unsynchronized'. At the time a slave Player is registered by
 * {@link #addSlave(Controller) addSlave()} method, synchronization status is 'Synchronizing'.
 * When synchronization is successfully established, the status is 'Synchronized'. In case that established
 * synchronization cannot be kept within given tolerance, the status returns to 'Synchronizing' and related
 * control objects try to re-synchronize automatically unless synchronization status is set to
 * <I>OUT_OF_SYNC_UNSYNCHRONIZABLE</I>.
 * <BR><BR>
 *
 * MasterSlaveSyncLinkageControl supports only simple Master/Slave relationship. That is,
 * the object accepts relationship between a single master Player and slave Players. It is not
 * allowed that a single JMF Player becomes a master Player and a slave Player at the same time.
 * (A potential master Player can be a slave Player.) In addition, a slave Player can have only a single
 * master Player. In case that an application tries to establish the relationship that violates these conditions,
 * <I>IllegalArgumentException</I> is thrown when {@link #addSlave(Controller) addSlave()} is called.
 * <BR><BR>
 *
 * Instance of MasterSlaveSyncLinkageControl can be obtained from a JMF Controller via the methods
 * getControl(String) and getControls(). A single Controller object of a JMF Player can create at most one
 * instance of MasterSlaveSyncLinkageControl.
 * <BR>
 * <BR>
 *
 * <B>Synchronization Status and Status Constants</B>
 * <BR>
 <table class=MsoTableGrid border=1 cellspacing=0 cellpadding=0
 style='border-collapse:collapse;border:none'>
 <tr>
  <td width=206 rowspan=2 valign=top style='width:154.45pt;border:solid windowtext 1.0pt;
  padding:0mm 5.4pt 0mm 5.4pt'>
  <p class=MsoNormal><span lang=EN-US>Unsynchronized</span></p>
  <p class=MsoNormal><span lang=EN-US>Master/Slave relationship is not
  established and Players can be running freely.</span></p>
  </td>
  <td width=206 rowspan=2 valign=top style='width:154.45pt;border:solid windowtext 1.0pt;
  border-left:none;padding:0mm 5.4pt 0mm 5.4pt'>
  <p class=MsoNormal><span lang=EN-US>Synchronizing</span></p>
  <p class=MsoNormal><i><span lang=EN-US>MASTERPLAYER_STOOPED,
  SLAVEPLAYER_STOPPED, OUT_OF_SYNC_MASTER_CLK, OUT_OF_SYNC_SLAVE_CLK,
  OUT_OF_SYNC_OTHERSs</span></i></p>
  </td>
  <td width=206 valign=top style='width:154.5pt;border:solid windowtext 1.0pt;
  border-left:none;padding:0mm 5.4pt 0mm 5.4pt'>
  <p class=MsoNormal><span lang=EN-US>Synchronized</span></p>
  <p class=MsoNormal><i><span lang=EN-US>IN_SYNC</span></i></p>
  </td>
 </tr>
 <tr>
  <td width=206 valign=top style='width:154.5pt;border-top:none;border-left:
  none;border-bottom:solid windowtext 1.0pt;border-right:solid windowtext 1.0pt;
  padding:0mm 5.4pt 0mm 5.4pt'>
  <p class=MsoNormal><span lang=EN-US>Unsynchronizable</span></p>
  <p class=MsoNormal><i><span lang=EN-US>OUT_OF_SYNC_UNSYNCHRONIZABLE</span></i></p>
  </td>
 </tr>
</table>
 * <BR><BR>
 */
public interface MasterSlaveSyncLinkageControl extends Control {

	/**
	 * #Fields
	 */

	/**
	 * Status representing synchronized
	 */
	public final int IN_SYNC = 0;
	/**
	 * Status representing out of sync and Unsynchronzable state
	 */
	public final int OUT_OF_SYNC_UNSYNCHRONIZABLE = 1;
	/**
	 * Status representing out of sync due to unstable clock of master Player
	 */
	public final int OUT_OF_SYNC_MASTER_CLK = 2;
	/**
	 * Status representing out of sync due to unstable clock of slave Player
	 */
	public final int OUT_OF_SYNC_SLAVE_CLK = 3;
	/**
	 * Status representing stop() method is applied to a master Player
	 */
	public final int MASTERPLAYER_STOPPED = 4;
	/**
	 * Status representing stop() method is applied to a slave Player
	 */
	public final int SLAVEPLAYER_STOPPED = 5;
	/**
	 * Status representing out of sync due to other reasons
	 */
	public final int OUT_OF_SYNC_OTHERS = 255;
	/**
	 * #Fields
	 */

	/**
	 * Adds a specified Player as a slave Player that synchronizes with a master Player. Offset value given as a
	 * Time object states offset of time position of the specified slave Player and the master Player.
	 * If the specified Player already establishes synchronization, this method does nothing.
	 * @param ctrl Controller object of a potential slave Player to synchronize with this master Player.
	 * @param t Time object to represent offset between master Player's clock and the specified Player.
	 * @throws InsufficientResourcesException If the operation cannot be completed due to a lack
	 * of system resources.
	 * @throws IncompatibleSynchronizableMediaTypeException If media type Player handles is incompatible
	 * to establish synchronization.
	 * @throws IllegalArgumentException If a specified Controller object already forms a master or slave Player.
	 */
	public void addSlave(Controller ctrl, Time t)
	throws InsufficientResourcesException, IncompatibleSynchronizableMediaTypeException,
	IllegalArgumentException;

	/**
	 * Adds a specified Player as a slave Player that synchronizes with a master Player. Offset value
	 * between master Player and a specified Player is implicitly given as zero in this method. If
	 * the specified Player already establishes synchronization, this method does nothing.
	 * @param ctrl Controller object of a potential slave Player to synchronize with this master Player.
	 * @throws InsufficientResourcesException If the operation cannot be completed due to a lack
	 * of system resources.
	 * @throws IncompatibleSynchronizableMediaTypeException If media type Player handles is incompatible
	 * to establish synchronization.
	 * @throws IllegalArgumentException If a specified Controller object already forms a master or slave Player.
	 */
	public void addSlave(Controller ctrl)
	throws InsufficientResourcesException, IncompatibleSynchronizableMediaTypeException,
	IllegalArgumentException;

	/**
	 * Removes a service component from the presentation. This removal should not affect any
	 * progress of MasterTimeBase or status of synchronization.
	 * @param component The locator representing an individual service component to remove from
	 * the presentation.
	 * @throws InvalidLocatorException If the specified locator does not reference a selectable
	 * service component.
	 * @throws IllegalArgumentException If the service component is not part
	 * of the set of services to which the SyncMediaSelectControl is restricted, or if the service
	 * component must be presented in conjunction with another service component that is not
	 * part of the current presentation.
	 * @throws NoMasterTimeBaseException If a service component which contains MasterTimeBase is removed.
	 * @throws java.lang.SecurityException If the caller does not have
	 * javax.tv.media.MediaSelectPermission(component) permission.

	public void remove(Locator component)
	throws InvalidLocatorException, IllegalArgumentException,
		NoMasterTimeBaseException, java.lang.SecurityException;
	*/

	/**
	 * Removes a specified slave Player. The specified Player turns self-running.
	 * This removal should not affect any progress of a master Player.
	 * @param ctrl Controller object of a potential slave Player to synchronize with this master Player.
	 * @throws IllegalArgumentException If the specified Controller object is not registered as a slave Player.
	 */
	public void removeSlave(Controller ctrl)
	throws IllegalArgumentException;
	/**
	 * Gets assigned tolerance value of synchronization for a specified slave Player.
	 * @param player JMF Player object from which assigned tolerance value is obtained.
	 * @return Value of tolerance for synchronization in terms of MPEG-TS PCR.

	public long getToleranceAsPCR(Player player);
	*/

	/**
	 * Gets slave Players added by addSlave method. Actual synchronization status of each
	 * slave Player is not considered in return value of this method.
	 * @return An array of Controller objects which are added by addSlave method.
	 */
	public Controller[] getSlaves();
	/**
	 * Sets offset value in time for a slave Player to the master Players. The value is given as PCR
	 * of MPEG-TS.
	 * @param player JMF Player object to set offset value between master Player.
	 * @param Offset Offset value in terms of MPEG-TS PCR.

	public void setOffset(Player player, long Offset);
	*/

	/**
	 * Gets currrent synchronization status of s specified slave Player.
	 * @param ctrl Controller objcet of a slave Player object to obtain current synchronization status.
	 * @return Synchronization status in field values.
	 * @see #IN_SYNC Fields
	 */
	public int getSyncStatus(Controller ctrl);

	/**
	 * Adds the specified SyncStateChangedEventListener to receive the events. SyncStateChangedEvent
	 * occurs when synchronization status between the master Player and any slave Player is changed. If
	 * l is null, no exception is thrown and no action is performed.
	 * @param l SyncStateChangedEvent listener.
	 */
	public void addSyncStateChangedEventListener(SyncStateChangedEventListener l);

	/**
	 * Removes a specified SyncStateChangedEventListener to receive the events. If
	 * l is null, no exception is thrown and no action is performed.
	 * @param l SyncStateChangedEvent listener.
	 */
	public void removeSyncStateChangedEventListener(SyncStateChangedEventListener l);

	/**
	 * Returns an array of all the SyncStateChangedEvent listeners registered on this control.
	 * @return An array of registered SyncStateChangedEvent listeners.
	 */
	public SyncStateChangedEventListener[] getSyncStateChangedEventListeners();

}
