package org.dvb.media;

import javax.media.Control;
import javax.media.Controller;
import javax.media.Time;

/**
 * SyncControl is a Control object for synchronization of a slave JMF Player.
 * SyncControl manipulates offset in time with a master Player and allowable
 * torelance
 * of synchronization. When SyncControl object is obtained from a Controller
 * object of a JMF Player by
 * getControl(String) or getControls() methods, the JMF Player is a potential
 * slave
 * Player. Such a JMF Player becomes a slave Player when the Player is
 * registered as
 * a slave Player by {@link MasterSlaveSyncLinkageControl#addSlave(Controller)
 * addSlave()} methods of MasterSlaveSyncLinkageControl. <BR>
 * <BR>
 * 
 * SyncControl supports only simple Master/Slave relationship. That is,
 * the object accepts relationship between a single master Player and slave
 * Players. It is not
 * allowed that a single JMF Player becomes a master Player and a slave Player
 * at the same time.
 * In addition, a slave Player can have only a single master Player. <BR>
 * <BR>
 * 
 * Instance of SyncControl can be obtained from a JMF Player via the methods
 * getControl(String) and getControls(). A single JMF Player can create at most
 * one instance
 * of SyncControl.
 */
public interface SyncControl extends Control {

   /**
    * #Fields
    */

   /**
    * Sets threshold to determine whether or not a specified slave Player
    * establishes synchronization.
    * In other words, an application allows error of synchronization within a
    * given parameter, t.
    * 
    * @param t
    *           Allowable tolerance of synchronization in terms of Time object.
    */
   public void setTolerance(Time t);

   /**
    * Sets threshold to determine whether or not a specified Player establishes
    * synchronization.
    * In other words, an application allows error of synchronization within a
    * given parameter in terms
    * of MPEG-TS PCR value.
    * 
    * @param player
    *           JMF Player object to which tolerance value is specified.
    * @param offet
    *           Allowable tolerance of synchronization in terms of MPEG-TS PCR.
    * 
    *           public void setTolerance(Player player, long offet);
    */

   /**
    * Gets assigned tolerance value of synchronization for a specified slave
    * Player.
    * 
    * @return Value of tolerance for synchronization in terms of Time object.
    */
   public Time getTolerance();

   /**
    * Gets assigned tolerance value of synchronization for a specified slave
    * Player.
    * 
    * @param player
    *           JMF Player object from which assigned tolerance value is
    *           obtained.
    * @return Value of tolerance for synchronization in terms of MPEG-TS PCR.
    * 
    *         public long getToleranceAsPCR(Player player);
    */

   /**
    * Gets offset value of a specified slave Player against master Player, by
    * Time object.
    * 
    * @return Offset value in terms of Time object.
    */
   public Time getOffset();

   /**
    * Gets offset value of a specified slave Player against master Player, as an
    * MPEG-TS PCR value.
    * 
    * @param player
    *           JMF Player object to obtain offset value.
    * @return Offset value in terms of MPEG-TS PCR.
    * 
    *         public long getOffsetAsPCR(Player player);
    */

   /**
    * Sets offset value in time for a slave Player to the master Player. The
    * value is given by
    * Time object.
    * 
    * @param t
    *           Offset value in terms of Time object.
    */
   public void setOffset(Time t);

   /**
    * Gets a JMF Player from which an instance of this Control is obtained.
    * 
    * @return Player object from which an instance of this Control is obtained.
    * 
    *         public Player getSourcePlayer();
    */

   /**
    * Gets a MasterSlaveSinkLinkageControl object that this SyncControl is
    * currently synchronizing with.
    * 
    * @return MasterSlaveSinkLinkageControl object that this SyncControl is
    *         currently synchronizing with.
    */
   public MasterSlaveSyncLinkageControl getMaster();

}
