package org.dvb.media.monitoring;

/**
 * Event is signalling a change in the buffer status.<BR>
 * <BR>
 * 
 * Events are sent when a JMF Player is in the Started state.<BR>
 * <BR>
 * 
 * To prevent implementation from being overloaded
 * {@link BufferingStatusEvent#BUFFERING_STATUS_UNDERRUN} and
 * {@link BufferingStatusEvent#BUFFERING_STATUS_BACK_TO_NORMAL} events should
 * not be sent faster than one per seconds.
 * 
 */
public class BufferingStatusEvent extends MonitoringEvent {

   private static final long serialVersionUID                = 5579982511981783878L;

   /**
    * Indicates that buffering status back to a normal state from
    * {@link BufferingStatusEvent#BUFFERING_STATUS_UNDERRUN} or
    * {@link #BUFFERING_STATUS_STOPPED} states.
    */
   public static final int   BUFFERING_STATUS_BACK_TO_NORMAL = 1;

   /**
    * Indicates that buffering status is underrun. It means that coding rate is
    * too high compared with network line rate.
    */
   public static final int   BUFFERING_STATUS_UNDERRUN       = 2;

   /**
    * Indicates that buffering has been stopped. It means that network
    * connection has been lost.
    */
   public static final int   BUFFERING_STATUS_STOPPED        = 3;

   private int               bufferingStatus;

   /**
    * Construct the event.
    * 
    * @param source
    *           the source of the event. The platform shall always pass in the
    *           JMF Player presenting the media whose property has been changed
    * @param bufferingStatus
    *           the status of the buffer
    * 
    * @throws IllegalArgumentException
    *            thrown if bufferingStatus is out of range
    * 
    */
   public BufferingStatusEvent(Object source, int bufferingStatus) throws IllegalArgumentException {
      super(source);
      if (bufferingStatus != BUFFERING_STATUS_BACK_TO_NORMAL && bufferingStatus != BUFFERING_STATUS_UNDERRUN
            && bufferingStatus != BUFFERING_STATUS_STOPPED) {
         throw new IllegalArgumentException("Buffering status out of range.");
      }
      this.bufferingStatus = bufferingStatus;
   }

   /**
    * Get buffering status.
    * 
    * @return status of the buffer
    */
   public int getBufferingStatus() {
      return bufferingStatus;
   }
}
