package org.dvb.media.monitoring;

/**
 * A MonitoringControl allows application to get information associated with the
 * Streamed
 * CoD being presented to the user. <BR>
 * <BR>
 * 
 * Additionally it allows application to control quality of
 * CoD by setting bitrate of downloaded CoD.
 * 
 */
public interface MonitoringControl extends javax.media.Control {

   /**
    * Add a listener for MonitoringEvent.
    * 
    * @param l
    *           the listener to add
    */
   public void addMonitoringListener(org.dvb.media.monitoring.MonitoringListener l);

   /**
    * Remove a listener for MonitoringEvent.
    * 
    * @param l
    *           the listener to remove
    */
   public void removeMonitoringListener(org.dvb.media.monitoring.MonitoringListener l);

   /**
    * Get all available qualities of streams in associated CoD. In case of video
    * CoD an array of {@link VideoStreamQualityInfo}s shall be
    * returned.
    * 
    * Precognitions:
    * Information is available after prefetching
    * 
    * @return all available qualities of associated CoD
    * 
    * @throws IllegalStateException
    *            if Player is neither in Prefetched nor Started state
    */
   public StreamQualityInfo[] getAvailableStreamQualities() throws IllegalStateException;

   /**
    * Get the current quality of associated CoD. It is the quality at which
    * associated CoD is being downloaded and presented to the user. In case of
    * video CoD an instance of {@link VideoStreamQualityInfo} shall
    * be
    * returned.
    * 
    * Precognitions:
    * Information is available after prefetching
    * 
    * @return quality of stream at which associated CoD is being
    *         downloaded and presented to the user
    * 
    * @throws IllegalStateException
    *            if Player is neither in Prefetched nor Started state
    */
   public StreamQualityInfo getStreamQuality() throws IllegalStateException;

   /**
    * Set initial quality of associated CoD. After Player starts, a quality
    * could be changed by Adaptive Streaming. <BR>
    * <BR>
    * 
    * Application can check CoD's available qualities using
    * {@link MonitoringControl#getAvailableStreamQualities()} method.
    * 
    * @param streamQuality
    *           the stream quality of CoD
    * 
    * @throws IllegalArgumentException
    *            thrown if CoD does not support given quality
    * @throws IllegalStateException
    *            if Player is not in the Prefetched state
    */
   public void setInitialStreamQuality(StreamQualityInfo streamQuality) throws IllegalArgumentException, IllegalStateException;

   /**
    * Set fixed quality of associated CoD. It allows application to force
    * downloading CoD with a given video resolution and stream bitrate, and thus
    * disable adaptive streaming support for associated CoD. <BR>
    * <BR>
    * 
    * Application can check CoD's available qualities using
    * {@link MonitoringControl#getAvailableStreamQualities()} method.
    * 
    * @param streamQuality
    *           the stream quality of CoD
    * @throws IllegalArgumentException
    *            thrown if CoD does not support given quality
    * @throws IllegalStateException
    *            if Player is not in the Prefetched state
    */
   public void setFixedStreamQuality(StreamQualityInfo streamQuality) throws IllegalArgumentException, IllegalStateException;

   /**
    * Get the current speed (in bits per second) with which the bits are sent
    * onto the wire.
    * 
    * @return current speed with which the bits are sent onto the wire
    */
   public long getCurrentLineRate();

   /**
    * Get the video format of associated CoD. <BR>
    * <BR>
    * 
    * Formats are encoded as MPEG-7 term IDs as extended by the DVB IPI
    * handbook, e.g. MPEG-2 video, main profile, main level is encoded as
    * "2.2.2".
    * 
    * Precognitions:
    * Information is available after prefetching
    * 
    * @return video format of the associated CoD or null if CoD does not have
    *         video stream
    * 
    * @throws IllegalStateException
    *            thrown if Player is neither in Prefetched nor Started state
    */
   public String getVideoFormat() throws IllegalStateException;

   /**
    * Get the audio format of the associated CoD. <BR>
    * <BR>
    * 
    * Formats are encoded as MPEG-7 term IDs as extended by the DVB IPI
    * handbook, e.g. MPEG-1 layer 3 audio is encoded as "3.3".
    * 
    * Precognitions:
    * Information is available after prefetching
    * 
    * @return audio format of the associated CoD
    * 
    * @throws IllegalStateException
    *            thrown if Player is neither in Prefetched nor Started state
    */
   public String getAudioFormat() throws IllegalStateException;
}
