package org.dvb.spi.ict;

import java.io.File;

/**
 * This interface represents a reference to a resource that will
 * be downloaded using an interaction channel transport service
 * provider.  It exposes a set of methods that correspond with
 * those in org.dvb.dsmcc.DSMCCObject, so that the applications
 * can use DSMCCObject to reference resources that will be
 * downloaded by the GEM implementation via the service provider
 * implementation.
 * <p>
 * Like DSMCCObject, ResourceTransportObject instances can
 * either be in a loaded or unloaded state.  When in a
 * loaded state, the contents of the resource can be accessed
 * either directly from memory as a byte array or else from a
 * physical file in a local file system (could also be a
 * RAM-based file system).
 * <p>
 * Note that unlike DSMCCObject instances, resource transport
 * objects are only used to download resources with associated
 * content (i.e. not for URIs that represent directories).
 */
public interface ResourceTransportObject {
    /**
     * Resource load type that indicates the file has been loaded in
     * memory as a byte array.
     */
    int LOAD_TYPE_MEMORY = 0;

    /**
     * Resource load type that indicates the file has been loaded to
     * a local physical file system.
     */
    int LOAD_TYPE_FILE = 1;

    /**
     * Return the URI identifying the resource to be downloaded.
     * <p>
     * @return String the URI for this resource
     */
    String getURI();

    /**
     * Add a listener to receive notifications of updates to this
     * resource.  Note that update notifications will not be fired
     * until after the object has succcessfully entered the loaded
     * state for the first time.
     * <p>
     * Once an object has successfully entered the loaded state
     * once, this event shall continue to be fired when changes are
     * detected regardless of further transitions in or out of the
     * loaded state.
     * <p>
     * @param listener the listener to be added
     */
    void addUpdateListener(ResourceUpdateListener listener);

    /**
     * Remove a listener that is currently receiving notifications
     * of updates to this resource.  If the specified listener has
     * not previously been added, then calling this method has no
     * effect.
     * <p>
     * @param listener the listener to be removed
     */
    void removeUpdateListener(ResourceUpdateListener listener);

    /**
     * Load the resource.  When this method returns, the resource
     * will have been succesfully downloaded. If the download fails,
     * then a ResourceLoadException will be thrown.
     * <p>
     * @throws ResourceLoadException
     */
    void synchronousLoad() throws ResourceLoadException;

    /**
     * Load the resource asynchronously. The specified listener will
     * be notified of the loading progress, including subsequent
     * successful completion or failure.
     * <p>
     * @param listener the listener to be notified
     */
    void asynchronousLoad(AsyncResourceLoadListener listener);

    /**
     * Return a flag indicating whether the resource has been
     * succesfully downloaded.
     */
    boolean isLoaded();

    /**
     * Return a flag indicating whether the metadata for this
     * resource (existence flag, content length, and MIME type) is
     * available for retrieval via the corresponding accessor
     * methods.  Depending on the specifics of the provider
     * implementation and transport mechanism, it may be that this
     * information is available without needing to call the
     * <code>loadMetadata</code> method; that method should be
     * called only when this method returns <code>false</code>.
     * 
     * @return boolean - Flag indicating whether the metadata for
     *         this resource has been loaded and is ready for
     *         retrieval via relevant accessor methods
     */
    boolean isMetadataLoaded();

    /**
     * Return a value indicating where the associated resource has
     * been downloaded.  Possible values include:
     * <p>
     * <ui><code>LOAD_TYPE_MEMORY</code> - indicates that the
     * resource was loaded into memory as a byte array, and can be
     * retrieved using the <code>getResourceBytes()</code> method.
     * <li><code>LOAD_TYPE_FILE</code> - indicates that the resource
     * was loaded to a file on a local physical file system, and can
     * be retrieved using the <code>getResourceFile()</code> method.
     * <p>
     * @return int - the load type
     * @throws IllegalStateException if the resource is not loaded
     */
    int getResourceLoadType();

    /**
     * Return the resource file containing the downloaded resource.
     * This method should only be called if the resource is loaded
     * and <code>getResourceLoadType()</code> returns
     * <code>LOAD_TYPE_FILE</code>.
     * <p>
     * The provider must ensure that the returned file is readable
     * by the GEM implementation.
     * <p>
     * @return File - the resource load file
     * @throws IllegalStateException if the resource is not loaded
     *                               or is loaded to a byte array in
     *                               memory
     */
    File getResourceFile();

    /**
     * Return a byte array containing the downloaded resource. This
     * method should only be called if the resource is loaded and
     * <code>getResourceLoadType()</code> returns
     * <code>LOAD_TYPE_MEMORY</code>.
     * <p>
     * @return byte [] the resource data
     * @throws IllegalStateException if the resource is not loaded
     *                               or is loaded to a file
     */
    byte [] getResourceBytes();

    /**
     * This method is used to abort a load in progress. It can be used
     * to abort either a synchronousLoad or an asynchronousLoad.
     * <p>
     * @throws IllegalStateException if load of the resource has not
     *                               already been started
     */
    void abort();

    /**
     * Tells the service provider that the downloaded resource
     * object is no longer used, and therefore any resources
     * allocated to this object can be freed.  This method puts the
     * ResourceTransportObject back into the unloaded state.
     * <p>
     * @throws IllegalStateException if the resource is not loaded
     */
    void unload();

    /**
     * Load the resource metadata (existence flag, content length,
     * and MIME type) for this resource.  Note that some providers
     * will be able to determine the metadata information without
     * downloading the resource content by using cached directory
     * lookup information, but in general this may not be the case.
     * In cases where the provider must download the resource to
     * find the directory information, this call may have an
     * equivalent effect to calling <code>asynchronousLoad()</code>.
     * If a load request is already in progress, this method may
     * also block until the outstanding load request is complete.
     * <p>
     * This method may fail either asynchronously with notification
     * event sent to the listener or else synchronously during this
     * method call.
     * <p>
     * Once the metadata is loaded, the following methods may be
     * called:
     * <p>
     * <ul>
     * <li><code>exists()</code>
     * <li><code>getMimeType()</code>
     * <li><code>getLength()</code>
     * </ul>
     * <p>
     * @param listener A listener to be notified of asynchronous
     *                 loading events.
     */
    void loadMetadata(AsyncResourceLoadListener listener) throws ResourceLoadException;

    /**
     * Return a flag indicating whether the resource exists.  Note
     * that some providers may be able to determine resource
     * existence without downloading the resource using cached
     * directory lookup information, but in general this may not be
     * the case. In cases where the provider must download the
     * resource to determine resource existence, this call may
     * result in a synchronous load operation being performed before
     * returning.
     * 
     * @return boolean
     */
    boolean exists();

    /**
     * Return the MIME type of the resource.  Note that some
     * providers may be able to determine the MIME type without
     * downloading the resource using cached directory lookup
     * information, but in general this may not be the case.  In
     * cases where the provider must download the resource to
     * determine the MIME type, this call may result in a
     * synchronous load operation being performed before returning.
     * 
     * @return String
     */
    String getMimeType();

    /**
     * Return the length in bytes of content of this resource.  Note
     * that some providers may be able to determine the size without
     * downloading the resource using cached directory lookup
     * informatino, but in general this may not be the case.  In
     * cases where the provider must download the resource to
     * determine the length, this call may result in a synchronous
     * load operation being performed before returning.  In
     * addition, since this information may be populated from a
     * directory, it is not guaranteed to be accurate; the most
     * authoritative information is specified after download from
     * either <code>getResourceBytes().length</code> or
     * <code>getResourceFile().length()</code>, depending on the
     * load type.
     * 
     * @return long the length in bytes of the resource content
     */
    long getLength();
}

