
package org.dvb.spi.si.simple;

import org.dvb.spi.SystemBoundProvider;
import org.dvb.spi.selection.ServiceReference;
import java.util.Date;
import java.util.List;


/** 
 * This interface describes a source of information about the schedules
 * and program events in a broadcast channel. 
 * <p>
 * This class and related classes and interfaces are intended to be used 
 * as follows:
 * <ul>
 * <li>When new program data is available, the SI source calls the 
 *     method on SimpleSIProviderListener to tell the platform which 
 *     entities have data available about them.
 * <li>The platform can, at any time, ask the provider for information 
 *	on certain entities. This may be in response to the provider 
 *	telling the platform that new information is available. It may 
 *	be in response to an application asking for information about an 
 *	entity whose description is not available in the terminal. 
 * </ul>
 * <p>
 * In this API, collections of data are represented with random-access
 * java.util.List type.
 * <p>
 * This class involves several methods that pass an instance of List
 * to the terminal.  Once a list instance has been give to the terminal,
 * it must not be changed.  The results of changing a list that has been
 * given to the terminal via this API are undefined, and may include
 * termination of the enclosing xlet.
 *
 * @see org.dvb.spi.SystemBoundProvider
 * @see org.dvb.spi.selection.SelectionProvider
 * @see java.util.List
 * @see java.util.ArrayList
 **/
public interface SimpleSIProvider extends SystemBoundProvider {

    /**
     * Called by the platform in order to obtain events from this provider.
     * @param l the listener to register
     **/
    public void registerListener(SimpleSIProviderListener l);

    /** 
     * Called by the platform when it no longer wishes to receive events
     * from this provider.
     * @param l the listener to remove
     **/
    public void unregisterListener(SimpleSIProviderListener l);
  
    /**
     * Called by the terminal to request program information from a
     * provider for a given service.
     * @param r the service for which program information is requested
     * @return references for all of the known programs for the given service.
     **/
    public ProgramReference[] getProgramReferences(ServiceReference r);

    /**
     * Called by the terminal to request schedule
     * information from a provider.
     * The output shall be returned in an array of the same size
     * and order as the input. If the schedule is not available on
     * any service listed in the input then the corresponding
     * entry in the results array shall be null.
     * 
     * @param services  references
     * identifying the services whose schedules are requested
     * 
     * @param startDate the starting date and time from which to 
     *  return schedule descriptions (inclusive)
     *
     * @param endDate the finishing date and time up to which 
     * schedule descriptons should be returned (exclusive)
     *
     * @return an array of schedule descriptions
     */
    public ScheduleDescription[] getScheduleDescriptions(
	    ServiceReference[] services,
	    Date startDate, Date endDate);

    /**
     * Called by a terminal to request program description
     * information from a provider.
     * The output shall be returned in an array of the same size
     * and order as the input. If the schedule is not available on
     * any service listed in the input then the corresponding
     * entry in the results array shall be null.
     *
     * @param programReferences references
     * identifying the programs whose schedules are requested
     *
     * @param preferredLanguages The preferred language(s) for 
     *     program names and descriptions.  This is a hint of
     *     the language(s) likely to be requested of the
     *	   descriptions themselves.
     *
     * @return an array of program descriptions
     */
    public ProgramDescription[] getProgramDescriptions(
	    ProgramReference[] programReferences,
	    String[] preferredLanguages);
}

