package org.dvb.tvanytime.metadata;

/**
* Class that holds the result of a database query. When a database query is completed the 
* DatabaseQueryResult will contain all the metadata that matches the query. 
* The document will be in the form of a TV Anytime compliant document 
* (i.e. the top level element will be a TVAMain, followed by all the programme/group/... tables)
* The {@link DatabaseQueryResult#getAvailableTables} method can be used to see what 
* tables (ProgramInformation, GroupInformation, etc) are in the result.
* @see Database 
* @see DatabaseQuery
* @see ResultListener 
*/
  public abstract interface DatabaseQueryResult   {

/**
* Returns a list of the metadata tables present in this result set. This method will return
* null until the database query has been completed and the {@link DatabaseQueryResult#isComplete} method
* returns true.
* @return an array of type {@link Table}, with one element for each table in the result. If there are no tables 
* in the result, an empty array is returned. For each element of the array the methods {@link Table#getSortCriteriaFieldIDs} 
* and {@link Table#getSortCriteriaOrderings} can be used to determine whether the results from a particular table 
* have been sorted. 
*/
  public  abstract org.dvb.tvanytime.metadata.Table[] getAvailableTables() ;

  
/**
* Get the root element of this result. This method will return
* null until the database query has been completed and the {@link DatabaseQueryResult#isComplete} method
* returns true.
* @return an element called "TVAMain", which has one child called "ProgramDescription",
* or null if the DatabaseQueryResult contains only ContentReferencingTable
* fragments.
*/
  public  abstract org.dvb.xml.jdom.Element getRootElement() ;

/**
* Get a ListIterator for all of the TV Anytime fragments in this result. This method will return 
* null until the database query has been completed and the {@link DatabaseQueryResult#isComplete} 
* method returns true. Each fragment is an object that implements the Element interface. For example a 
* fragment for the description of one TV programme would be an element with the name "ProgramInformation".
* Note that these fragments do <em>not</em> contain TVAMain etc and therefore calling <code>getParent</code> on 
* a fragment will return null.
* @return a ListIterator of objects implementing the org.dvb.xml.jdom.Element interface
*/
  public  abstract org.dvb.tvanytime.metadata.ListIterator getFragments() ;

/**
* Get a ListIterator for the specified TV Anytime fragment types that are found in this result. 
* This method will return null until the database query has been completed and the {@link DatabaseQueryResult#isComplete} 
* method returns true. Each fragment is an object that implements the Element interface. For example a 
* fragment for the description of one TV programme would be an element with the name "ProgramInformation".
* Note that these fragments do <em>not</em> contain TVAMain etc and therefore calling <code>getParent</code> on 
* a fragment will return null.
* @param fragmentNames the list of required fragments. Each item is a String 
* that contains the name of the top level Element of the fragment, for 
* example "ProgramInformation" to request ProgramInformation fragments.
* @return a ListIterator of objects implementing the org.dvb.xml.jdom.Element interface
*/
  public  abstract org.dvb.tvanytime.metadata.ListIterator getFragments(java.lang.String[] fragmentNames) ;
  
/**
* Indicates that the database query has been completed. Database queries can take time to be processed. As 
* individual fragments in the results become available they will be signalled by the databaseResultAdded 
* method of {@link ResultListener}. The completion of the database query will be signalled by the notifyFinished 
* method of {@link ResultListener} after which the isComplete method will return true.
* @return true if this DatabaseQueryResult is complete.
*/
  public  abstract boolean isComplete() ;

/**
* Returns the status of this DatabaseQueryResult at completion. The value returned by this method before the 
* database query has been completed and the {@link DatabaseQueryResult#isComplete} method returns true is unspecified.
* 
* @return a value chosen from {@link ResultListener#REQUEST_COMPLETED}, {@link ResultListener#REQUEST_FAILED}, 
* {@link ResultListener#RESPONSE_INCOMPLETE} and {@link ResultListener#RESPONSE_TRUNCATED}
*/
  public  abstract int getStatus() ;
  
/**
* Closes the result and releases all system resources, removes all listeners. 
* This function should be called when an application no longer requires this 
* DatabaseQueryResult. Once close() has been called, an application must not 
* call any methods in this object.
*/
  public  abstract void close() ;

/**
 * Get the exception that caused {@link #getStatus} to return <code>REQUEST_FAILED</code> or 
 * <code>RESPONSE_INCOMPLETE</code>.
  *
 * @return The exception that caused the failure. The exception returned will be a
 * {@link org.dvb.tvanytime.metadata.DatabaseException}.
 *
 * @throws IllegalStateException if {@link #getStatus} does not return {@link ResultListener#REQUEST_FAILED} or
 * {@link ResultListener#RESPONSE_INCOMPLETE}
 */
  public abstract Exception getFailedException() throws java.lang.IllegalStateException;

  /**
   * Get a ListIterator of the unknown fragments that are found in this result.
   * An unknown fragment is one with a <code>DVBContextPath</code> value that is not
   * recognised by the resident decoder. If there are no unknown fragments this method will return null.
   *
   * Each of the returned fragments is represented by an {@link org.dvb.xml.jdom.UnknownElementContent}
   * object. The {@link org.dvb.xml.jdom.UnknownElementContent#readBits} method of these objects will 
   * access the <code>FragmentUpdatePayload</code> of each fragment.
   *
   * @return a ListIterator of {@link org.dvb.xml.jdom.UnknownElementContent} objects or null.
   */
  public abstract ListIterator getUnknownFragments();

  /**
   * Get a ListIterator of unknown fragments with the specified <code>DVBContextPath</code>
   * that are found in this result.
   *
   * An unknown fragment is one with a <code>DVBContextPath</code> value that is not
   * recognised by the resident decoder. If the specified <code>DVBContextPath</code> value
   * is one supported by the resident decoder this method will return null. If
   * the value is not supported but there are no matching fragments an empty
   * ListIterator will be returned.
   *
   * Each of the returned fragments is represented by an {@link org.dvb.xml.jdom.UnknownElementContent}
   * object. The {@link org.dvb.xml.jdom.UnknownElementContent#readBits} method of these objects will 
   * access the <code>FragmentUpdatePayload</code> of each fragment.
   *
   * @param DVBContextPath the <code>DVBContextPath</code> of the required fragments.
   * @return a ListIterator of {@link org.dvb.xml.jdom.UnknownElementContent} objects.
   */
  public abstract ListIterator getUnknownFragments(int DVBContextPath);

}


