package org.dvb.tvanytime.metadata;

/**
* Sources of metadata conforming to this interface can be continuously monitored for changes.
*
* Queries are constructed using a {@link DatabaseQuery} object, which is then passed to
* different database instances depending on the required source.
* Results are returned inside a {@link DatabaseQueryResult} object which is initially empty.
* An application can receive a sequence of events as each element is added by providing an instance
* of {@link ResultListener}.
* The scope of the metadata that is returned by a query can be restricted by the use of
* {@link QueryConstraints}.
* An application can receive notification of metadata changes that affect the {@link DatabaseQueryResult} 
* by providing an instance of {@link MetadataChangedListener}.
* 
* @see Database
* @see DatabaseQuery
* @see ResultListener
* @see MetadataChangedListener
* @see QueryConstraints
* @see DatabaseQueryResult
*/
public abstract interface MonitoredDatabase extends Database  {

  /**
   * Get all available metadata on a specific CRID. 
   * @param crid The CRID to look up
   * @param resultListener an instance of ResultListener or null
   * @param changeListener an instance of a metadataChangedListener
   * @return DatabaseQueryResult that contains metadata fragments where the given CRID 
   * is the primary key. 
   * The document will be in the form of a TV Anytime compliant document 
   * (i.e. the top level element will be a TVAMain, followed by all the programme/group/... tables). 
   * The {@link  DatabaseQueryResult#getAvailableTables} method can be used to see if this 
   * contains a ProgramInformation or GroupInformation table, which can be used to decide
   * if this is a group or programme CRID. If the CRID cannot be found, an empty document 
   * will be returned.
   * 
   * <p>Implementations can choose whether or not to inline credits information 
   * directly, or to reference a fragment in the CreditsInformationTable.
   * 
   * @throws DatabaseException if the CRID is invalid or if there is an error 
   * accessing the database. 
   */
     public  abstract org.dvb.tvanytime.metadata.DatabaseQueryResult getMetadata(
         org.dvb.tvanytime.resolution.CRID crid,
         org.dvb.tvanytime.metadata.ResultListener resultListener, 
        org.dvb.tvanytime.metadata.MetadataChangedListener changeListener
     ) throws org.dvb.tvanytime.metadata.DatabaseException;

   /**
   * Get all available metadata on a specific CRID. 
   * @param crid The CRID to look up
   * @param resultListener an instance of ResultListener or null
   * @param changeListener an instance of a metadataChangedListener
   * @param constraints the constraints to apply to this metadata request 
   * @return DatabaseQueryResult that contains metadata fragments where the given CRID 
   * is the primary key. 
   * The document will be in the form of a TV Anytime compliant document 
   * (i.e. the top level element will be a TVAMain, followed by all the programme/group/... tables).
   * The {@link  DatabaseQueryResult#getAvailableTables} method can be used to see if this 
   * contains a ProgramInformation or GroupInformation table, which can be used to decide
   * if this is a group or programme CRID. If the CRID cannot be found, an empty document 
   * will be returned.
   * 
   * <p>Implementations can choose whether or not to inline credits information directly, 
   * or to reference a fragment in the CreditsInformationTable. Specifying the 
   * CreditsInformationTable in the QueryConstraints determines if the server returns 
   * credits information at all, but does not impact the above choice on how to include 
   * the credits information. The implementation will not return credits information if 
   * the CreditsInformationTable is not specified in the QueryConstraints. 
   *  
   * @throws DatabaseException if the CRID is invalid or if there is an error 
   * accessing the database. 
   */
     public  abstract org.dvb.tvanytime.metadata.DatabaseQueryResult getMetadata(
         org.dvb.tvanytime.resolution.CRID crid,
         org.dvb.tvanytime.metadata.ResultListener resultListener,
        org.dvb.tvanytime.metadata.MetadataChangedListener changeListener,
         org.dvb.tvanytime.metadata.QueryConstraints constraints
     ) throws org.dvb.tvanytime.metadata.DatabaseException;

     /**
     * Make a metadata query to the database. 
     * @param query The query to perform on the database 
     * @param resultListener an instance of ResultListener or null
     * @param changeListener an instance of a metadataChangedListener
     * @return DatabaseQueryResult that contains all the metadata that matches the 
     * query. The document will be in the form of a TV Anytime compliant document 
     * (i.e. the top level element will be a TVAMain, followed by all the 
     * programme/group/... tables) If nothing matches the query, an empty document 
     * will be returned.
     * 
     * <p>Implementations can choose whether or not to inline credits information directly, 
     * or to reference a fragment in the CreditsInformationTable.
     * 
     * <p>A ServiceInformationTable will always be included in the response if a 
     * ServiceInformation entry is being referenced (using a serviceIDRef) from a 
     * ProgramLocationTable.
     * 
     * @throws DatabaseException if the CRID is invalid or if there is an error 
     * accessing the database. 
     */
       public  abstract org.dvb.tvanytime.metadata.DatabaseQueryResult query(
           org.dvb.tvanytime.metadata.DatabaseQuery query,
           org.dvb.tvanytime.metadata.ResultListener resultListener,
            org.dvb.tvanytime.metadata.MetadataChangedListener changeListener
       ) throws org.dvb.tvanytime.metadata.DatabaseException;

     /**
     * Make a metadata query to the database. 
     * @param query The query to perform on the database 
     * @param resultListener an instance of ResultListener or null
     * @param changeListener an instance of a metadataChangedListener
     * @param constraints the constraints to apply to this metadata request 
     * @return DatabaseQueryResult that contains all the metadata that matches the 
     * query. The document will be in the form of a TV Anytime compliant document 
     * (i.e. the top level element will be a TVAMain, followed by all the 
     * programme/group/... tables) If nothing matches the query, an empty document 
     * will be returned.
     * 
     * <p>Implementations can choose whether or not to inline credits information directly, 
     * or to reference a fragment in the CreditsInformationTable. Specifying the 
     * CreditsInformationTable in the QueryConstraints determines if the server returns 
     * credits information at all, but does not impact the above choice on how to include 
     * the credits information. The implementation will not return credits information if 
     * the CreditsInformationTable is not specified in the QueryConstraints.
     * 
     * <p>A ServiceInformationTable will always be included in the response if a 
     * ServiceInformation entry is being referenced (using a serviceIDRef) from a 
     * ProgramLocationTable. Thus, specifying the ServiceInformationTable in the 
     * QueryConstraints will have no effect if a ProgramLocationTable is being returned.
     * 
     * @throws DatabaseException if the CRID is invalid or if there is an error 
     * accessing the database. 
     */
       public  abstract org.dvb.tvanytime.metadata.DatabaseQueryResult query(
           org.dvb.tvanytime.metadata.DatabaseQuery query,
           org.dvb.tvanytime.metadata.ResultListener resultListener, 
           org.dvb.tvanytime.metadata.MetadataChangedListener changeListener,
           org.dvb.tvanytime.metadata.QueryConstraints constraints
       ) throws org.dvb.tvanytime.metadata.DatabaseException;
  
}



