package org.dvb.tvanytime.resolution;

import org.davic.net.Locator;

/**
* <p>This class provides access to Content Referencing Information.
* Asynchronous methods supporting CRID resolution provide resolution results 
* in the form of a <code>ResolutionResponse</code> object which encapsulates 
* Content Locators or Content References with state data.
* 
* Where the resolution source is not specified explicitly, or more than one 
* resolution source is specified, the sources are searched in the following 
* order until resolution is complete:
* 
* <p>1. <code>RESOLVE_LOCAL</code>
* <br>2. <code>RESOLVE_DVB_NO_TUNING</code>
* <br>3. <code>RESOLVE_DVB_WITH_TUNING</code>
* <br>4. <code>RESOLVE_IP</code>
* 
* <p>If resolution can be completed using from information held locally in a 
* transparent cache then listeners will be informed immediately.
*/

public abstract class ContentReferencingManager {

  public ContentReferencingManager() { }

  /**
   * Get a ContentReferencingManager where the platform chooses the context and resolution provider preferences.
   * @return a ContentReferencingManager
   */
  public static org.dvb.tvanytime.resolution.ContentReferencingManager getInstance(
  ) {return (null);}

  /**
   * Get a ContentReferencingManager for a specified context where the platform chooses the resolution provider preferences.
   * @param contextId the contextId as specified in the Resolution Provider Notification Table (RNT) defined in ETSI TS 102 323
   * @param contextIdType the contextIdType
   * @return a ContentReferencingManager or null for receivers without a DVB tuner
   */
  public static org.dvb.tvanytime.resolution.ContentReferencingManager getInstance(int contextId, int contextIdType
  ) {return (null);}
  
  /**
   * Get a ContentReferencingManager for a specified context and resolution provider. The context and resolution provider
   * are specified by the supplied {@link ResolutionProviderInformation}.
   * @param provider the {@link ResolutionProviderInformation}
   * @return a ContentReferencingManager
   */
  public static org.dvb.tvanytime.resolution.ContentReferencingManager getInstance(ResolutionProviderInformation provider) {return (null);}
  
  /**
   * Get a ContentReferencingManager which uses the specified server for IP-based queries. If used for non IP-based queries the
   * platform will choose the context and resolution provider preferences.
   * @param URL the address of the IP server
   * @return a ContentReferencingManager
   */
  public static org.dvb.tvanytime.resolution.ContentReferencingManager getInstance(Locator URL) {return(null);}
    
  /**
   * Used to resolve a <code>CRID</code>. It is preferable to use 
   * <code>resolveCRIDList()</code> where possible. All the specified resolution 
   * sources are searched in the order described above until resolution is completed.
   * This method will not attempt to tune if the resolutionSources does not include 
   * <code>RESOLVE_DVB_OTHER</code>.
   *  
   * @param cr The query <code>CRID</code> we are trying to resolve. 
   * @param resolutionSources A bitwise OR of <code>RESOLVE_LOCAL</code>, 
   * <code>RESOLVE_DVB_ACTUAL</code> etc that specifies which sources of resolution information to use.
   * @param listener Holds a reference to the client class of the callback 
   * function, allowing us to call the appropriate callback when resolution is 
   * complete. 
   */
  public abstract void resolveCRID(
      org.dvb.tvanytime.resolution.CRID cr,
      int resolutionSources,
      org.dvb.tvanytime.resolution.ResolutionListener listener
  );

  /**
   * Used to resolve a <code>CRID</code>. It is preferable to use 
   * <code>resolveCRIDList()</code> where possible. All available resolution 
   * sources are searched in the order described above until resolution is completed.
   * @param cr The query <code>CRID</code> we are trying to resolve.
   * @param listener Holds a reference to the client class of the callback 
   * function, allowing us to call the appropriate callback when resolution is 
   * complete. 
   */
  public abstract void resolveCRID(
      org.dvb.tvanytime.resolution.CRID cr,
      org.dvb.tvanytime.resolution.ResolutionListener listener
  );

  /**
   * <p>Used to resolve a list of <code>CRID</code>s. This function should be 
   * used in preference to <code>resolveCRID</code> as it permits a more 
   * efficient implementation for resolution over IP or from broadcast. For IP 
   * it enables multiple resolution queries to be bundled into a single HTTP 
   * request. In the broadcast case it means that tuning to a new service and 
   * waiting for the location resolution information to be repeated does not 
   * have to be done for each and every <code>CRID</code>. <p>Note that the 
   * number of callbacks generated will be exactly equal to the length of the 
   * list. i.e. there is a separate callback for each <code>CRID</code>. 
   * @param crList The list of query Content References we are trying to 
   * resolve. 
   * @param listener Holds a reference to the client class of the callback 
   * function, allowing us to call the appropriate callback when resolution is 
   * complete. 
   */
  public abstract  void resolveCRIDList(
      org.dvb.tvanytime.resolution.CRID[] crList,
      org.dvb.tvanytime.resolution.ResolutionListener listener
  ) ;

  /**
   * <p>Used to resolve a list of <code>CRID</code>s.  All the specified resolution 
   * sources are searched in the order described above until resolution is completed.
   * The method will not attempt to tune if the resolutionSources does not include 
   * <code>RESOLVE_DVB_OTHER</code>. This function should be used in preference to
   * <code>resolveCRID</code> as it permits a more 
   * efficient implementation for resolution over IP or from broadcast. For IP 
   * it enables multiple resolution queries to be bundled into a single HTTP 
   * request. In the broadcast case it means that tuning to a new service and 
   * waiting for the location resolution information to be repeated does not 
   * have to be done for each and every <code>CRID</code>. <p>Note that the 
   * number of callbacks generated will be exactly equal to the length of the 
   * list. i.e. there is a separate callback for each <code>CRID</code>. 
   * @param crList The list of query Content References we are trying to 
   * resolve. 
   * @param resolutionSources A bitwise OR of <code>RESOLVE_LOCAL</code>, 
   * <code>RESOLVE_DVB_ACTUAL</code> etc that specifies which sources of resolution information to use. 
   * @param listener Holds a reference to the client class of the callback 
   * function, allowing us to call the appropriate callback when resolution is 
   * complete. 
   */
  public  abstract void resolveCRIDList(
      org.dvb.tvanytime.resolution.CRID[] crList,
      int resolutionSources,
      org.dvb.tvanytime.resolution.ResolutionListener listener
  ) ;

  /**
   * Get a set of {@link ResolutionProviderInformation} for the specified resolution sources. 
   * @param resolutionSources A bitwise OR of <code>RESOLVE_LOCAL</code>, <code>RESOLVE_DVB_ACTUAL</code> etc that 
   * specifies which sources of resolution information to include in the returned information.
   *  
   * @return an array of {@link ResolutionProviderInformation}
   */
  public static ResolutionProviderInformation[] getResolutionProviderInformation (
      int resolutionSources
  ) { return null;}

  /**
   * Determine whether this ContentReferencingManager provides a complete resolution service for the specified CRID authority.
   * 
   * @param CRIDAuthority the CRID Authority
   * @return true if a complete resolution service is provided 
   */
  public abstract boolean isComplete(String CRIDAuthority) ;
      
  /**
   * Constant value for <code>resolutionSources</code> which determines whether locators for locally recorded content should be returned when resolving.
   */
  public static final  int RESOLVE_LOCAL = 1;

  /**
   * Constant value for <code>resolutionSources</code> which determines whether DVB streams should be used when resolving but without tuning a reserved tuner.
   */
  public static final  int RESOLVE_DVB_NO_TUNING = 2;

  /**
   * Constant value for <code>resolutionSources</code> which determines whether DVB streams should be used when resolving with tuning.
   */
  public static final  int RESOLVE_DVB_WITH_TUNING = 4;

  /**
   * Constant value for <code>resolutionSources</code> which determines whether IP servers should be used when resolving. 
   */
  public static final  int RESOLVE_IP = 8;

  /**
   *  Constant value for <code>contextIdType</code> as defined in ETSI TS 102 323, indicating that the contextId is a bouquet_id.
   */
  public final static short BOUQUET_ID                = 0x00;
  
  /**
   *  Constant value for <code>contextIdType</code> as defined in ETSI TS 102 323, indicating that the contextId is an original_network_id.
   */
  public final static short ORIG_NET_ID               = 0x01;
  
  /**
   *  Constant value for <code>contextIdType</code> as defined in ETSI TS 102 323, indicating that the contextId is a network_id.
   */
  public final static short NETWORK_ID                = 0x02;
  
}



