package org.dvb.tvanytime.resolution;

/**
 * An interface encapsulting a set of Resolution Authority Records (RARs) for one Service 
 * Provider. This is the information carried in one iteration of the Resolution Provider loop 
 * of the RNT (except that the RARs may have been filtered based on the required resolution 
 * sources). RARs which require the scheduled download of CRI data are not included unless 
 * the platform has successfuly downloaded a valid data set.
 * 
 * @see ContentReferencingManager
 */

public interface ResolutionProviderInformation {

  /**
   * Get the contextId of this ResolutionProviderInformation.
   * @return the contextId or -1 if this is unspecified
   */
  public int getContextId();
	
  /**
   * Get the contextIdType of this ResolutionProviderInformation.
   * @return the contextIdType or -1 if this is unspecified
   */
  public int getContextIdType();
  
  /**
   * Get the resolution sources supported by this ResolutionProviderInformation. 
   * 
   * @return a bitwise OR of {@link ContentReferencingManager#RESOLVE_DVB_NO_TUNING}, 
   * {@link ContentReferencingManager#RESOLVE_DVB_WITH_TUNING} and {@link ContentReferencingManager#RESOLVE_IP} 
   * 
   */
  public int getResolutionSources();
	
  /**
   * Get the name of the Resolution Provider.
   * @return the Resolution Provider Name
   */
  public String getResolutionProviderName();

  /**
   * List the CRID Authorities supported by this ResolutionProviderInformation.
   * @return an array of Strings carrying CRID Authority names
   */
  public String[] getCRIDAuthorities();
  
  /**
   * List the <code>CRID_authority_policy</code> fields for the CRID Authorities supported by this 
   * ResolutionProviderInformation, as signalled in the RNT.
   * @return an array carrying the value of the <code>CRID_authority_policy</code> field for each CRID authority.
   */
  public int[] getCRIDAuthorityPolicies();
  
  /**
   * Test whether this ResolutionProviderInformation references a complete set of CRI data for the specified CRID Authority. 
   * @param CRIDAuthority the CRID Authority of interest
   * @return true if a complete set of CRI data is referenced
   */
  public boolean isComplete(String CRIDAuthority);

}


