package org.dvb.tvanytime.resolution;

/**
* <p>Interface for encapsulating a resolution response. A resolution function 
* call will return an object which implements this interface <p>All the 
* searching through the local database and possibly multiple IP servers is 
* done transparently. <p>There are 4 states that an application needs to 
* distinguish: <ol> <li>The resolution is successful AND complete requiring 
* no further resolution [status = "resolved" and complete = "true"]</li> 
* <li>There are children in the response but it is incomplete and needs 
* further resolution [status = "resolved" and complete = "false"]</li> 
* <li>Resolve again at a certain time but there are no children 
* [status = "cannot yet resolve" (or "unable to resolve" for a secondary resolution source)]</li>
* <li>PDR should discard the CRID because it is invalid 
* [status = "discard CRID" (or "unable to resolve" for a primary resolution source)]</li></ol>
* <p><table border=on> <tr> <th></th><th>1</th><th>2</th><th>3</th><th>4</th> </tr> <tr align="center"> 
* <td><code>isValid()</code></td><td>true</td><td>true</td><td>true</td> 
* <td>false</td> </tr> <tr align="center"> 
* <td><code>isSuccessful()</code></td><td>true</td><td>true</td><td>false</td> 
* <td>undefined</td> </tr> <tr align="center"> 
* <td><code>isCompletelyResolved()</code></td><td>true</td><td>false</td> 
* <td>false</td><td>undefined</td> </tr> <tr align="center"> 
* <td><code>getChildren()</code></td><td>Vector</td><td>Vector</td><td>null</td> 
* <td>null</td> </tr> <tr align="center"> 
* <td><code>getNextResolutionDate()</code></td><td>null</td><td>Date or null</td> 
* <td>Date or null</td><td>null</td> </tr> <tr align="center"> 
* <td><code>getType()</code></td><td><code>TYPE_GET_ALL</code> or <code>TYPE_ALTERNATIVE</code> 
* </td><td><code>TYPE_GET_ALL</code> or <code>TYPE_ALTERNATIVE</code></td><td>undefined</td> 
* <td>undefined</td> </tr> </table> <p>Note that resolution 
* authorities are never returned as a possible response. The redirection 
* mechanism is automatically followed. In other words, a client of this class 
* does not need to know about Resolving Authority Records, nor the various 
* Resolution handlers that are used to carry out the resolution. 
*/
public abstract interface ResolutionResponse   {

/**
* Used to indicate whether or not a valid response has been obtained.
* This method will return false if the resolution status is signalled as "discard CRID"
* or if the resolution source is primary source and the resolution status is signalled
* as "unable to resolve".
* @return true if a valid response has been obtained. 
*/
  public  abstract boolean isValid(
  ) ;

/**
* Used to indicate whether or not the Content Reference was successfully 
* resolved. (i.e. whether or not the response has children)
* @return true if there is no need to resolve again 
*/
  public  abstract boolean isSuccessful(
  ) ;

/**
* Used to indicate whether or not further resolution is required. 
* @return true if there is no need to resolve again 
*/
  public  abstract boolean isCompletelyResolved(
  ) ;

/**
* Returns the Content Reference that was used in the query which created this 
* response. This is necessary since resolution results may not be returned in the 
* order they were requested.
* @return a <code>CRID</code> object 
*/
  public  abstract org.dvb.tvanytime.resolution.CRID getQueryCRID(
  ) ;

/**
* This method provides access to the resolution results. 
* By using the <code>instanceof</code> operator the client can determine the 
* type of object that each element of the vector contains. 
* @return a vector of CRIDs or a vector of ContentLocations. Null if there are no 
* children.
*/
  public  abstract java.util.Vector getChildren(
  ) ;

/**
* Returns true if this response is for a group CRID that has CRIDs as its 
* children. If <code>hasCRIDChildren</code> returns <code>true</code>, then the <code>getChildren</code> call will 
* return a Vector that only contains CRID objects. If <code>hasCRIDChildren</code> returns 
* <code>false</code>, the <code>getChildren</code> function will return a Vector of ContentLocation 
* objects, or the <code>getChildren</code> call will return an empty Vector. 
* @return true if the query CRID is known to be a group CRID (i.e. has CRID 
* children) 
*/
  public  abstract boolean hasCRIDChildren(
  ) ;

/**
* If the resolution was not successful, or successful but incomplete,
* this function may indicate when an application should try to resolve again. 
* The Date returned shall contain a copy of the date contained in this instance
* such that it can be freely modified by applications without changing the date
* contained in this instance.
* @return the date of the next resolution. Null if no re-resolve date is 
* signalled.
*/
  public  abstract java.util.Date getNextResolutionDate(
  ) ;

/**
* Signals how the list of children should be handled. 
* @return an integer taking value <code>TYPE_GET_ALL</code> or 
* <code>TYPE_ALTERNATIVE</code> 
*/
  public  abstract int getType(
  ) ;
  
/**
* Get the source of this resolution response.
* @return a value which is either <code>RESOLVE_LOCAL</code>, <code>RESOLVE_DVB</code>, or <code>RESOLVE_IP</code> as defined 
* in the <code>ContentReferencingManager</code> class that identifies the source of this 
* resolution response.
*/
public abstract int getSource();

  /**
   * Value returned by <code>getType</code> which indicates that all the children should be captured.
   * e.g. this would be used for a series. 
   */
  public static final  int TYPE_GET_ALL = 0;

  /**
   * Value returned by <code>getType</code> which indicates that just one of the children should be captured. 
   * e.g. this would be used for a programme and its repeat. 
   */
  public static final  int TYPE_ALTERNATIVE = 1;

}


